<?php
/*
Plugin Name: Buy All Products
Description: Adds a button to add a bundle of products to the cart with a URL suffix.
Version: 1.2
Author: EasyToolsWP.com
*/

// Security check
if ( ! defined( 'ABSPATH' ) ) {
    exit;
}

// Register plugin settings
function buy_all_register_settings() {
    add_option('buy_all_min_products', '1'); // Default to 1
    add_option('buy_all_max_products', '15'); // Default to 15
    add_option('buy_all_enable_categories', '1');
    add_option('buy_all_enable_tags', '1');
    add_option('buy_all_url_suffix', '');
    add_option('buy_all_button_color', '#007cba'); // Default button color
    add_option('buy_all_text_color', '#ffffff'); // Default text color
    register_setting('buy_all_options_group', 'buy_all_min_products');
    register_setting('buy_all_options_group', 'buy_all_max_products');
    register_setting('buy_all_options_group', 'buy_all_enable_categories');
    register_setting('buy_all_options_group', 'buy_all_enable_tags');
    register_setting('buy_all_options_group', 'buy_all_url_suffix');
    register_setting('buy_all_options_group', 'buy_all_button_color');
    register_setting('buy_all_options_group', 'buy_all_text_color');
}
add_action('admin_init', 'buy_all_register_settings');

// Add a settings page in WooCommerce admin menu
function buy_all_plugin_menu() {
    add_submenu_page('woocommerce', 'Buy All Settings', 'Buy All Settings', 'manage_options', 'buy-all-settings', 'buy_all_settings_page');
}
add_action('admin_menu', 'buy_all_plugin_menu');

// Settings page
function buy_all_settings_page() {
    ?>
    <div class="wrap">
        <h1>Buy All Settings</h1>
        <form method="post" action="options.php">
            <?php settings_fields('buy_all_options_group'); ?>
            <?php do_settings_sections('buy_all_options_group'); ?>
            <table class="form-table">
                <tr valign="top">
                    <th scope="row">Minimum products for display</th>
                    <td><input type="text" name="buy_all_min_products" value="<?php echo esc_attr(get_option('buy_all_min_products')); ?>" /></td>
                </tr>
                <tr valign="top">
                    <th scope="row">Maximum products for display</th>
                    <td><input type="text" name="buy_all_max_products" value="<?php echo esc_attr(get_option('buy_all_max_products')); ?>" /></td>
                </tr>
                <tr valign="top">
                    <th scope="row">Enable on categories</th>
                    <td><input type="checkbox" name="buy_all_enable_categories" value="1" <?php checked(1, get_option('buy_all_enable_categories')); ?> /></td>
                </tr>
                <tr valign="top">
                    <th scope="row">Enable on tags</th>
                    <td><input type="checkbox" name="buy_all_enable_tags" value="1" <?php checked(1, get_option('buy_all_enable_tags')); ?> /></td>
                </tr>
                <tr valign="top">
                    <th scope="row">URL suffix</th>
                    <td><input type="text" name="buy_all_url_suffix" value="<?php echo esc_attr(get_option('buy_all_url_suffix')); ?>" /></td>
                </tr>
                <tr valign="top">
                    <th scope="row">Button color</th>
                    <td><input type="text" name="buy_all_button_color" value="<?php echo esc_attr(get_option('buy_all_button_color')); ?>" class="my-color-field" /></td>
                </tr>
                <tr valign="top">
                    <th scope="row">Text color</th>
                    <td><input type="text" name="buy_all_text_color" value="<?php echo esc_attr(get_option('buy_all_text_color')); ?>" class="my-color-field" /></td>
                </tr>
            </table>
            <?php submit_button(); ?>
        </form>
    </div>
    <?php
}

// Enqueue color picker
function buy_all_enqueue_color_picker($hook_suffix) {
    // First check that $hook_suffix is appropriate for your admin page
    if ('woocommerce_page_buy-all-settings' !== $hook_suffix) {
        return;
    }
    wp_enqueue_style('wp-color-picker');
    wp_enqueue_script('buy-all-script-handle', plugins_url('buy-all-script.js', __FILE__), array('wp-color-picker'), false, true);
}
add_action('admin_enqueue_scripts', 'buy_all_enqueue_color_picker');

// Add the button to the header next to sorting options
function buy_all_add_header_button() {
    if (is_product_category() || is_product_tag()) {
        $min_products = get_option('buy_all_min_products', 1);
        $max_products = get_option('buy_all_max_products', 15);
        $enable_categories = get_option('buy_all_enable_categories', 1);
        $enable_tags = get_option('buy_all_enable_tags', 1);
        $url_suffix = get_option('buy_all_url_suffix', '');
        $button_color = get_option('buy_all_button_color', '#007cba');
        $text_color = get_option('buy_all_text_color', '#ffffff');

        if ((!$enable_categories && is_product_category()) || (!$enable_tags && is_product_tag())) {
            return;
        }

        $term = get_queried_object();
        $taxonomy = is_product_category() ? 'product_cat' : 'product_tag';

        // Get the number of products in the current category or tag
        $term_info = get_term_by('id', $term->term_id, $taxonomy);
        $product_count = $term_info ? $term_info->count : 0;

        if ($product_count < $min_products || $product_count > $max_products) {
            return;
        }

        $product_ids = implode(',', get_posts(array(
            'post_type' => 'product',
            'posts_per_page' => -1,
            'fields' => 'ids',
            'tax_query' => array(
                array(
                    'taxonomy' => $taxonomy,
                    'field' => 'id',
                    'terms' => $term->term_id
                )
            )
        )));

        ?>
        <style>
            .buy-all-header-button {
                display: inline-block;
                background-color: <?php echo esc_attr($button_color); ?>;
                color: <?php echo esc_attr($text_color); ?>;
                padding: 10px 20px;
                border: none;
                border-radius: 5px;
                cursor: pointer;
                margin-left: 20px;
                box-shadow: 0px 4px 6px rgba(0, 0, 0, 0.1);
                transition: box-shadow 0.3s ease-in-out;
				float:right;
            }
            .buy-all-header-button:hover {
				background-color:<?php echo esc_attr($button_color); ?>;
                box-shadow: 0px 8px 15px rgba(0, 0, 0, 0.3);
            }
            .woocommerce-result-count {
                display: none;
            }
        </style>
        <button id="buy-all-header-button" class="buy-all-header-button"
                data-product-ids="<?php echo esc_attr($product_ids); ?>" 
                data-suffix="<?php echo esc_attr($url_suffix); ?>">
            Buy all <?php echo $product_count; ?> now!
        </button>
        <script>
            document.addEventListener('DOMContentLoaded', function() {
                document.getElementById('buy-all-header-button').addEventListener('click', function() {
                    const productIds = this.getAttribute('data-product-ids').split(',');
                    const suffix = this.getAttribute('data-suffix');
                    
                    // Confirmation
                    if (confirm('Are you sure you want to add all these products to the cart?')) {
                        const addToCartUrl = '<?php echo admin_url('admin-ajax.php'); ?>';

                        const formData = new FormData();
                        formData.append('action', 'add_bundle_to_cart');
                        formData.append('product_ids', JSON.stringify(productIds));
                        formData.append('suffix', suffix);

                        fetch(addToCartUrl, {
                            method: 'POST',
                            body: formData
                        })
                        .then(response => response.json())
                        .then(data => {
                            if (data.success) {
                                window.location.href = '<?php echo wc_get_cart_url(); ?>' + suffix;
                            } else {
                                alert('Failed to add items to cart.');
                            }
                        })
                        .catch(error => {
                            console.error('Error:', error);
                        });
                    }
                });
            });
        </script>
        <?php
    }
}
add_action('woocommerce_before_shop_loop', 'buy_all_add_header_button');

// AJAX function to add products to cart
function buy_all_add_bundle_to_cart() {
    if (!isset($_POST['product_ids'])) {
        wp_send_json_error('No products specified.');
        return;
    }

    $product_ids = json_decode(stripslashes($_POST['product_ids']), true);
    if (empty($product_ids)) {
        wp_send_json_error('Invalid product IDs.');
        return;
    }

    foreach ($product_ids as $product_id) {
        $product_id = (int) $product_id;
        if ($product_id > 0) {
            WC()->cart->add_to_cart($product_id);
        }
    }

    wp_send_json_success();
}
add_action('wp_ajax_add_bundle_to_cart', 'buy_all_add_bundle_to_cart');
add_action('wp_ajax_nopriv_add_bundle_to_cart', 'buy_all_add_bundle_to_cart');
